--[[

]]--
local addonName, vars = ...
local L = vars.L --- localizzazione
local ERA = _G[addonName]

--Upvalue
local CreateFrame, strlen = CreateFrame,strlen

--
--legge un valore d'appoggio settato in un frame
function ERA:GetFrameVal(frame, val)
	--if not frame._custom then return end
	if frame.ERA and frame.ERA._custom then
		return frame.ERA._custom[val]   
	end
end

--setta un valore d'appoggio in un frame
function ERA:SetFrameVal(frame, var, val)
	frame.ERA = frame.ERA or {}
	frame.ERA._custom = frame.ERA._custom or {}
	frame.ERA._custom[var] = val
end

function ERA:GetFrameDB(frame, val)  
	local r = ERA:DB()
	local class = ERA:GetFrameVal(frame, "class")
	local type = ERA:GetFrameVal(frame,"type") or 1
	if r[class] and r[class][type] then
		return r[class][type][val]   
	end   
end

function ERA:SetFrameDB(frame, var, val)
	local r = ERA:DB()
	local class = ERA:GetFrameVal(frame, "class")
	local type = ERA:GetFrameVal(frame,"type") or 1
	if r[class] and r[class][type] then
		r[class][type][var] = val
	end   
end

function ERA:EnableFrameDB(frame)
	local r = ERA:DB()
	local class = ERA:GetFrameVal(frame, "class")
	if not class then return end
	local type = ERA:GetFrameVal(frame,"type") or 1
	if r[class] and r[class][type] then
		r[class][type] = r[class][type] or {}
		ERA:SetFrameVal(frame, "DBEnabled", true)
	end      
end

function ERA:SaveFramePos(frame)
	if not ERA:GetFrameVal(frame, "DBEnabled") then return end
	local from, _, to, x, y = frame:GetPoint()   
	ERA:SetFrameDB(frame, "from", from)
	ERA:SetFrameDB(frame,"to", to)
	ERA:SetFrameDB(frame,"x",x)
	ERA:SetFrameDB(frame,"y", y)
   
end

function ERA:RestoreFramePos(frame)
   local from, parent, to, x, y = ERA:GetFrameDB(frame, "from"),frame:GetParent(),ERA:GetFrameDB(frame,"to"), ERA:GetFrameDB(frame,"x"), ERA:GetFrameDB(frame,"y")
   if from and parent and to and x and y then
      frame:ClearAllPoints()
      frame:SetPoint(from, parent, to, x, y)
   end   
end

function ERA:LockFrame(frame)
	ERA:SetFrameVal(frame,"lock", true)
	ERA:SetFrameDB(frame,"lock",true)
	frame.title.lock:SetTexture(ERA:GetTexture("lock"))
    frame:EnableMouse()
    frame.title.lockFrame:EnableMouse(true)	
	frame.bg:SetAlpha(0)
end

function ERA:UnlockFrame(frame)
	ERA:SetFrameVal(frame,"lock")
	ERA:SetFrameDB(frame,"lock")
	frame.title.lock:SetTexture(ERA:GetTexture("unlock"))
    frame:EnableMouse(true)
	frame.bg:SetAlpha(1)
end

function ERA:LockUnlockFrame(frame,state)	
	if state then
		ERA:LockFrame(frame)
	else      
		ERA:UnlockFrame(frame)
	end   
end


--*********************************************
--*********************************************
local colors ={
	r = {1,0,0},
	g = {0,1,0},
	b = {0,0,1},	
	y = {1,1,0},
	c = {0.4,0.8,0.9},
	n = {0,0,0},
	w = {1,1,1},
}

setmetatable(colors, {__index=function()return {1,1,0}end,})

local function GetFrameVal(frame, val)
	--if not frame._custom then return end
	
	if frame.ERA and frame.ERA._custom then		
		return frame.ERA._custom[val]   
	end
end

--setta un valore d'appoggio in un frame
local function SetFrameVal(frame, var, val)
	frame.ERA = frame.ERA or {}
	frame.ERA._custom = frame.ERA._custom or {}
	frame.ERA._custom[var] = val	
end

local function GetFrameDB(frame, val)  
	local r = ERA:DB()
	local class = GetFrameVal(frame, "class")
	local type = GetFrameVal(frame,"type") or 1
	if r[class] and r[class][type] then
		return r[class][type][val]   
	end   
end

local function SetFrameDB(frame, var, val)	
	local r = ERA:DB()
	local class = GetFrameVal(frame, "class")
	local type = GetFrameVal(frame,"type") or 1
	if r[class] and r[class][type] then				
		r[class][type][var] = val
	end   
end

local function EnableFrameDB(frame)
	local r = ERA:DB()
	local class = GetFrameVal(frame, "class")	
	if not class then return end
	local type = GetFrameVal(frame,"type") or 1	
	if r[class] and r[class][type] then
		r[class][type] = r[class][type] or {}
		SetFrameVal(frame, "DBEnabled", true)
	end      
end

local function SaveFramePos(frame)	
	if not GetFrameVal(frame, "DBEnabled") then return end
	local from, _, to, x, y = frame:GetPoint()  
	SetFrameDB(frame, "from", from)
	SetFrameDB(frame,"to", to)
	SetFrameDB(frame,"x",x)
	SetFrameDB(frame,"y", y)
   
end

local function RestoreFramePos(frame)
   local from, parent, to, x, y = GetFrameDB(frame, "from"),frame:GetParent(),GetFrameDB(frame,"to"), GetFrameDB(frame,"x"), GetFrameDB(frame,"y")
   if from and parent and to and x and y then
      frame:ClearAllPoints()
      frame:SetPoint(from, parent, to, x, y)
   end   
end

local function LockFrame(frame)
	SetFrameVal(frame,"lock", true)
	SetFrameDB(frame,"lock",true)
	frame.title.lock:SetTexture(ERA:GetTexture("lock"))
    frame:EnableMouse()
    frame.title.lockFrame:EnableMouse(true)	
	frame.bg:SetAlpha(0)
end

local function UnlockFrame(frame)
	SetFrameVal(frame,"lock")
	SetFrameDB(frame,"lock")
	frame.title.lock:SetTexture(ERA:GetTexture("unlock"))
    frame:EnableMouse(true)
	frame.bg:SetAlpha(1)
end

local function LockUnlockFrame(frame,state)	
	if state then
		LockFrame(frame)
	else      
		UnlockFrame(frame)
	end   
end

local frameProto = {
	movable = true,
	resizable = true,
	savePos = nil,
	title = true,
	lockable = true,
	closable = true,		
}

function frameProto:IsClosable()
	return self.closable
end

function frameProto:IsLockable()
	return self.lockable
end

function frameProto:GetTitle(txt)
	return self.frame.title.txt:GetText()
end

function frameProto:SetTitle(txt,color)	
	self.frame.title.txt:SetFormattedText("%s",txt)
	self.frame.title.txt:SetTextColor(unpack(colors[color]))
	local width = strlen(txt)*7+20+ (self:IsClosable() and 20 or 0) + (self:IsLockable() and 20 or 0)--7 è per la grandezza del font, andrebbe cambiato sulla base dell'effettiva grandezza
	self.frame:SetWidth(max(self.frame:GetWidth(), width))
end

function frameProto:SetFrameVal(var, val)	
	SetFrameVal(self.frame, var,val)
end

function frameProto:GetFrameVal(var)
	return GetFrameVal(self.frame, var)
end

function frameProto:EnableFrameDB()
	EnableFrameDB(self.frame)
end

function frameProto:RestoreFramePos()
	RestoreFramePos(self.frame)
end

function frameProto:LockUnlockFrame(state)
	LockUnlockFrame(self.frame,state)
end

function frameProto:SaveFramePos()
	SaveFramePos(self.frame)
end

function frameProto:RestoreFramePos()
	RestoreFramePos(self.frame)
end

function frameProto:SetFrameDB(var, val)
	SetFrameDB(self.frame, var, val)
end

function frameProto:GetFrameDB(var)
	return GetFrameDB(self.frame, var)
end

function frameProto:RestoreLock()
	LockUnlockFrame(self.frame, self:GetFrameDB("lock"))
end

function ERA:Frame(pTable)
	ptable = pTable or {}			
	if pTable.name then
		if type(pTable.name) ~= "string" and pTable.class then
			pTable.name = format("%s%s%s", addonName, pTable.class, pTable.type or 1)
		end
	end
	local frame = CreateFrame('Frame',pTable.name,UIParent)  
	frame.ERA = setmetatable({},{__index = frameProto,})
	frame.ERA.frame = frame
	--i valori della tabella passata	
	for k, v in pairs(pTable or {}) do		
		frame.ERA[k] = v
	end
	-----
	if frame.ERA.class then			
		frame.ERA:SetFrameVal("class", frame.ERA.class)
		if frame.ERA.type then			
			frame.ERA:SetFrameVal("type", frame.ERA.type)
		else
			frame.ERA.type = "1"
			frame.ERA:SetFrameVal("type", "1")
		end
	end				
	
	frame:SetSize(120,200)
	frame:SetPoint("TOP",UIParent, "TOP", 0, -50)
	frame.bg = frame:CreateTexture(nil, "BACKGROUND",nil,-7)
	frame.bg:SetAllPoints()
	frame.bg:SetTexture(0,0,0,0.2)
	frame:EnableMouse(true)
	frame:SetMovable(true)
	frame:SetResizable(true)
	frame:SetClampedToScreen(true)
	frame:SetFrameStrata("HIGH")

	if frame.ERA.savePos then			
		if frame.ERA:GetFrameVal("class") then			
			frame.ERA:EnableFrameDB()			
			frame.ERA:RestoreFramePos()
		else
			frame.ERA.savePos = nil
		end
	end
	
	if frame.ERA.movable then 
		frame:RegisterForDrag("LeftButton")
		frame:SetScript("OnDragStart", function(self)
			 if self:IsMovable() then
				self:StartMoving()
			 end
		end)
		frame:SetScript("OnDragStop", function(self)
			 self:StopMovingOrSizing() 
			 frame.ERA:SaveFramePos()
		end)
		frame:SetScript("OnHide", function()
			 frame:StopMovingOrSizing()       
			 frame.ERA:SaveFramePos()
		end)
	end
	
	if frame.ERA.title then
		frame.title = CreateFrame("Frame",nil,frame)
		frame.title:SetSize(1,20)
		frame.title:SetPoint("TOPRIGHT",frame,"TOPRIGHT")
		frame.title:SetPoint("TOPLEFT",frame,"TOPLEFT")
		frame.title.bg = frame.title:CreateTexture(nil,"BACKGROUND",nil,-7)
		frame.title.bg:SetAllPoints()
		frame.title.bg:SetTexture(1,1,1,0.4)	
		
		if frame.ERA.lockable then
			frame.title.lockFrame = CreateFrame("Frame",nil,frame.title)
			frame.title.lockFrame:SetSize(20,1)
			frame.title.lockFrame:SetPoint("TOPLEFT",frame.title,"TOPLEFT")		
			frame.title.lockFrame:SetPoint("BOTTOMLEFT",frame.title,"BOTTOMLEFT")		
			frame.title.lockFrame:EnableMouse(true)
			frame.title.lockFrame:SetScript("OnMouseUp", function(self)         
					ERA:LockUnlockFrame(frame, not ERA:GetFrameVal(frame,"lock"))
				end)
			
			frame.title.lock = frame.title.lockFrame:CreateTexture(nil,"BACKGROUND",-6)
			frame.title.lock:SetAllPoints()
			frame.title.lock:SetTexture(ERA:GetTexture("unlock"))
			frame.ERA:RestoreLock()			
		end
			
		frame.title.txt = frame.title:CreateFontString(nil, "BACKGROUND")
		frame.title.txt:SetFontObject(GameFontNormal)
		frame.title.txt:SetTextColor(0, 1, 0)
		frame.title.txt:SetPoint("TOPRIGHT",frame.title,"TOPRIGHT")
		frame.title.txt:SetPoint("BOTTOMRIGHT",frame.title,"BOTTOMRIGHT")
		frame.title.txt:SetPoint("TOPLEFT",frame.title,"TOPLEFT")
		if type(frame.ERA.title) == "string" then
			frame.ERA:SetTitle(frame.ERA.title,frame.ERA.titleColor)
		end

		frame.txt = frame:CreateFontString(nil, "BACKGROUND")
		frame.txt:SetFontObject(GameFontNormal)
		frame.txt:SetTextColor(1, 1, 1)
		frame.txt:SetPoint("TOPLEFT",frame.title,"BOTTOMLEFT",0,-5)   
		frame.txt:SetPoint("TOPRIGHT",frame.title,"BOTTOMRIGHT",0,-5)   
		frame.txt:SetWordWrap(true)
		
		if frame.ERA.closable then
			frame.title.close = CreateFrame("Button",nil, frame, "UIPanelCloseButton")
			frame.title.close:SetSize(20,1)
			frame.title.close:SetPoint("TOPRIGHT",frame.title,"TOPRIGHT")		
			frame.title.close:SetPoint("BOTTOMRIGHT",frame.title,"BOTTOMRIGHT")		
		end
	else	
		frame.ERA.lockable = nil
		frame.ERA.closable = nil
	end
	  
	return frame
end